/*---------------------------------------------------------*/
/*                     PATT   PACKAGE                      */
/*                                                         */
/*           aux.c:   auxiliary routines                   */
/*---------------------------------------------------------*/

#include <string.h>
#include <ctype.h>
#include "patt.h"
#include "io.h"

/*---------------------------------------------------------------*/
/* Gets bit from packed array, given location in packed ptr. form. */
/*---------------------------------------------------------------*/
char get_pack_bit(pack_type *address, struct PACK_PTR *loc)
{
	pack_type mask = 1 << (PACK_SIZE - 1 - (loc->bit));
	return( (address[loc->word] & mask) > 0 );
}

/*---------------------------------------------------------------*/
/* Sets bit in packed array, given location in packed ptr. form. */
/*---------------------------------------------------------------*/
void set_pack_bit(pack_type *address, struct PACK_PTR *loc, int value)
{
	pack_type mask = 1 << (PACK_SIZE - 1 - (loc->bit));
	if (!value) address[loc->word] &= (~mask);
	else address[loc->word] |= mask;
}

/*---------------------------------------------------------------*/
/* Increments a packed pointer to point to the next bit. */
/*---------------------------------------------------------------*/
void pack_incr(struct PACK_PTR *pack)
{
	(pack->bit)++;
	if ((pack->bit)<PACK_SIZE) return;
	pack->bit=0;
	(pack->word)++;
}

/*---------------------------------------------------------------*/
/* set_path: sets the path for input ASCII files */
/*---------------------------------------------------------------*/
void set_path(char *path)
{
        char *f;
	int i=0;
	char *I="set_path";

	while ( isspace(path[i]) ) i++;
        if ( path[i]=='\0' )
	  file_path[0] = '\0';
	else
	  {
	    while( path[i]!='\0' ) i++;
	    strcpy(file_path,path);
	    if ( file_path[i-1] != '/' ) file_path[i++] = '/';
	    file_path[i] = '\0';
	  }
	log_in(INFO,I,"path set to: %s",file_path);
}

/*---------------------------------------------------------------*/
/* name_format: makes a string all lowercase and removes blanks*/
/*---------------------------------------------------------------*/
void name_format( char *in, char *out )
{
    int j=-1,k=0;

    while( in[++j] != '\0' )
      if (!isspace(in[j])) out[k++] = lowercase(in[j]);
    out[k] = '\0';
}

/*---------------------------------------------------------------*/
/* lowercase: returns the lower case equivalent of a character*/
/*---------------------------------------------------------------*/
char lowercase( char c )
{
    if ( (c>='A') && (c<='Z') )
      return( 'a' + c - 'A' );
    else
      return(c);
}
/*---------------------------------------------------------------*/
/* pat_reset: reset package */
/*---------------------------------------------------------------*/
void patt_reset(void)
{
	chn_clear();
	out_clear();
	bs_clear();
}

/*-----------------------------------------------------------------*/
/* Fill i_hold bit to go come on just before end of current pattern*/
/*-----------------------------------------------------------------*/
void sample_and_hold(void)
{
  int i,j,max=0,id,zero_level;
  char *I="sample_and_hold";
  unsigned long mask1,mask0;

/* find longest pattern channel */
  for (j=0;j<MAX_CHANNELS;j++) if ( (i=output_level[j]) > max ) max=i;

/* find id of i_hold channel */
  if ( (id=chn_id("i_hold"))==-1 )
    {
      log_in(WARNING,I,"can't find i_hold channel");
      return;
    }

  mask1 = 1 << (id+1);
  mask0 = ~mask1;
  
/* fill 90% of channel with zeros */
  if (max>100) zero_level = max - 40;
  else zero_level = max - (max/10);

  for (i=0;i<zero_level;i++) output[2*i] &= mask0;
 
/* fill remainder with 1's */
  for (i=zero_level;i<max;i++) output[2*i] |= mask1;

  return;
}


