/*---------------------------------------------------------*/
/*                     PATT   PACKAGE                      */
/*                                                         */
/* bs.c:  For management of bitstreams in memory           */
/*---------------------------------------------------------*/
/* 4/11/96 mgs: added "store" bitsream */
/* 4/26/96 mgs: added argument to clock "store" or not */
/* 7/18/96 mgs: chnged to use logger rouinres instead of printf */
/* 1/26/98 mgs: fixed bs_getini_change bug for changing single bits */

#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "io.h"
#include "patt.h"

/*------------------------------------------------------------------*/
/* Load bitstreams from a .pat file into memory */
/*------------------------------------------------------------------*/
int bs_loadmem(char *in_name)
{
        char name[MAX_NAME_LEN], filename[120], *f;
	FILE *fp;
	int ch_num,i,j;
	char ch_name[MAX_NAME_LEN],c;
	struct BS_CONTENTS *temp,*temp2;
	struct PACK_PTR loc;
	char *I="bs_loadmem";

/* Standardize name */
        name_format( in_name, name );

/* Check that it does not already exist */
        if ( bs_id(name)!=-1 )
	  {
	    log_in(WARNING,I,"name %s already exists. Ignored",name);
	    return(-2);
	  }

/* Compose file name */
        filename[0] = '\0';
        f = strcpy( filename, file_path );
        f = strcat( filename, name );
        f = strcat( filename, ".pat"); 

#ifndef QUIET
        log_in(INFO,I,"reading data from %s",filename);
#endif


/* Open file */
	if ((fp=fopen(filename,"r"))==NULL)
	  {
	     log_in(WARNING,I,"could not open %s",filename);
	     return(1);
	   }

/* Allocate memory for first record  and set local pointers*/
	bs_fileindex[next_file].first=\
           (struct BS_CONTENTS *)malloc(sizeof(struct BS_CONTENTS));
	temp=bs_fileindex[next_file].first;
	temp2=temp;

/* Begin loop over characters in ASCII file */
	while (!feof(fp))
	  {

/* skip leading blanks in current line */
	     do  c=getc(fp); while ( isspace(c) );
             if ( feof(fp) || (c=='\n') ) continue;

/* if first character is a ! then line is a comment. Ignore.*/
	     if (c=='!') 
	       {
                  while ( (c!='\n') && !feof(fp) ) c=getc(fp);
                  continue;
		}

/* Line is not a comment */
/* Read off channel name (end of name marked by colon) */
		i=0;
                do {
                    if ( c == '\n' )
   		      {
		         log_in(WARNING,I,"missing colon. Data skipped");
		         fclose(fp);
                         return(2);
		       }
                    else if (!isspace(c)) ch_name[i++]=c;
                    c = getc(fp);
                 } while ( c != ':' && !feof(fp) );
                 ch_name[i] = '\0';
               
/* Test if the channel name has been defined */
		if ((ch_num=chn_id(ch_name))<0)
		  {
		    log_in(WARNING,I,"unknown channel name %s",ch_name);
		    fclose(fp);
                    return(3);
		  }

/* Put channel number in record for this line */
	        temp->channel=ch_num;

/* Pack bits into data field of record for this line*/
		loc.word=0;
		loc.bit=0;
		(temp->nBits)=0;
                c = getc(fp);
		while ( c != '\n' && !feof(fp) )
		  {
                        if (c=='_')
			    {
				set_pack_bit(temp->data,&loc,0);
				pack_incr(&loc);
				(temp->nBits)++;
			     }
			else if (!isspace(c))
			    {
				set_pack_bit(temp->data,&loc,1);
				pack_incr(&loc);
				(temp->nBits)++;
			     }
                        c = getc(fp);
		     }

/* check that line was not empty */
	     	 if (!(temp->nBits))
		    {
			log_in(WARNING,I,"empty bitstream.");
			fclose(fp);
                        return(4);
		    }

/* allocate memory for new record and advance local pointers */
	  temp->next=(struct BS_CONTENTS *)malloc(sizeof(struct BS_CONTENTS));
	  temp2=temp;
	  temp=temp->next;
	}


/* Success. Store bitstream name, clean up and return */
	free(temp);
	if (temp==temp2)
	  {
	    log_in(WARNING,I,"no bitstreams scanned from %s",filename);
	    return(5);
	  }
	temp2->next=NULL;
        f = strcpy( bs_fileindex[next_file].name, name ); 
	next_file++;
	fclose(fp);
	return(0);
}

/*------------------------------------------------------------------*/
/* Generate memory bitstreams from a .ini file                      */
/*                                                                  */
/* 07/11/97: routine just calls bs_getini_change.                   */
/*------------------------------------------------------------------*/
int bs_getini(char *in_name, int store)
{

  int i;

  i = bs_getini_change(in_name, store, 0, NULL, NULL);

  return(i);
}


/*----------------------------------------------------------------*/
/* Fill a given channel with a loop of nl times a given pattern */
/*----------------------------------------------------------------*/
struct BS_CONTENTS *bs_fillup\
(struct BS_CONTENTS *empty_record, char *chan, int nl, char *val)
/* val is a string representeng the pattern to be repeated 
   for example "0010111".
   The argument *empty_record is a pointer to an empty record at the 
   tail of a linked list. The returned pointer indicates the last
   filled record */
{
  struct PACK_PTR loc;
  struct BS_CONTENTS *old, *new;
  int i,j,jb,il=0,ch,nrec,nlrec,nbrec;
  int bits[64];
  char *I="bs_fillup";

/* find channel number */
  if ( (ch=chn_id(chan))<0 )
    {
      log_in(WARNING,I,"undefined channel %s not filled",chan);
      return(old);
    }

/*decode pattern*/
  jb=0;
  while ( val[jb]!='\0' ) 
    {
      bits[jb] = (val[jb]=='1');
      jb++;
    }
  if (jb==0)
    {
      log_in(WARNING,I,"unspecified fill pattern");
      return(old);
    }

/* let each record hold up to 75 bits. How many patterns/record? */
  nlrec = 75/jb;

/* how many bits per record?*/
  nbrec = nlrec * jb;

/* how many records? */
  nrec = nl/nlrec;

/*loop over records*/
  il = 0; 
  old = empty_record;
  new = old;
  for (j=0;j<=nrec;j++)
    {
       loc.word=0;
       loc.bit=0;
       new->nBits=0;
       new->channel = ch;

/* transfer next 75 or less bits */
       while (il<nl && new->nBits<nbrec)
	  {
	    for (i=0;i<jb;i++)
	      {
                 set_pack_bit(new->data,&loc,bits[i]);
	         pack_incr(&loc);
	        (new->nBits)++;
	       }
	    il++;
          }      

/* Allocate next data record */
        old = new;
       	old->next=\
         (struct BS_CONTENTS *)malloc(sizeof(struct BS_CONTENTS));
	new=old->next;
     }
/* finished. Return pointer to last full record. temp contains
   pointer to next empty record */
  return(old);
}

/*----------------------------------------------------------------*/
/* Return array index of given bitstream name  */
/*----------------------------------------------------------------*/
int bs_id(char *bs_name)
{
	int i;
	char bs_temp[MAX_NAME_LEN];
      	name_format(bs_name,bs_temp);
	for (i=0;i<next_file;i++)
		if (!strcmp(bs_temp,bs_fileindex[i].name)) return(i);
	return(-1);
}

/*----------------------------------------------------------------*/
/* Transfer bitstream to output pattern array */
/*----------------------------------------------------------------*/
int bs_output(char *bs_name)
{
	struct PACK_PTR p1;
	struct BS_CONTENTS *temp;
	int id,j,ch,level_check;
        char ibit;
        unsigned char chdone[MAX_CHANNELS]={0};
	unsigned long mask0,mask1;
	char *I="bs_output";

	if ( (id=bs_id(bs_name)) == -1 ) 
	  return(-1);

/* Get first data record  and note output_level for that channel
   All channels we are adding bits to must be at the same output 
   level  
*/
	temp=bs_fileindex[id].first;
        level_check = output_level[temp->channel];

/* Loop over all records and fill destination array */
	while (temp!=NULL)
	  {
	    p1.word=0;
	    p1.bit=0;
	    ch=temp->channel;

	    mask1 = 1 << (ch+1);
	    mask0 = ~mask1;

/*check starting level*/
            if ( (!chdone[ch]) && (output_level[ch]!=level_check) ) 
              {
                 log_in(WARNING,I,"problem found in channel %d",ch);
                 log_in(WARNING,I,"output level=%d, should be %d",\
			output_level[ch],level_check);
		 chn_print();
                 log_in(ERROR,I,"corrupted pattern: channels at different",\
			"levels");
	       }
            chdone[ch] = 1;

/*transfer bits in this record*/
	    for (j=0;j<(temp->nBits);j++) 
              {           
                  ibit = get_pack_bit(temp->data,&p1);
                  pack_incr( &p1 );
		  if (ibit)
		    output[2*output_level[ch]] |= mask1;
		  else
		    output[2*output_level[ch]] &= mask0;
		  output_level[ch]++;
 	       }
               
/*get next record*/
	    temp=temp->next;
	   }
	return(0);
}

/*----------------------------------------------------------------*/
/* Clear bitstreams from memory */
/*----------------------------------------------------------------*/
void bs_clear(void)
{
	int i;
	struct BS_CONTENTS *temp,*temp2;
  
	for (i=0;i<next_file;i++)
	{
	    temp=bs_fileindex[i].first;
	    while (temp)
	    {
		temp2=temp->next;
		free(temp);
		temp=temp2;
	    }
	    bs_fileindex[i].first = NULL;
	}
	next_file=0;
	return;
}
/*---------------------------------------------------------------*/
/* Print list of bitstream sets in memory */
/*---------------------------------------------------------------*/
void bs_print_index(void)
{
	int i;
	printf("\nBitstream files currently in memory: \n");
	for (i=0;i<next_file;i++)
		printf("slot %3d: %s\n",i,bs_fileindex[i].name);
	return;
}

/*---------------------------------------------------------------*/
/* Print contents of a bitstream set to the screen. */
/*---------------------------------------------------------------*/
void bs_print_contents(int id)
{
        struct BS_CONTENTS *temp=bs_fileindex[id].first;
	struct PACK_PTR p1;
	int i,j;
	
	printf("Contents of bitstream set %s\n",bs_fileindex[id].name);
	while (temp!=NULL)
		{
		printf("channel %d: \n",temp->channel);
		p1.word=0;
		p1.bit=0;
		for (i=0;i<(temp->nBits);i++)
			{
			j=get_pack_bit(temp->data,&p1);
			if (j==0) putchar('0');
			else putchar('1');
			pack_incr(&p1);
			}
		putchar('\n');
		temp=temp->next;
		}
	return;
}

/*---------------------------------------------------------------*/
/* Fill specified channels (or all if none specified) with zeros */
/* up to the level of the longest bitsream in the memory pattern */
/*---------------------------------------------------------------*/
void bs_pad_level(char *list)
{
  int i,j,top,lvl,valu;
  char *I="bs_pad_level";
  unsigned long mask;

  if ( list[0] == '\0' ) 
    {

/* Find level of longest channel */
      top = 0;
      for (j=0;j<MAX_CHANNELS;j++) 
	if (output_level[j]>top) top = output_level[j];

/* Fill all other channels to this level, looping over channels...*/
      for (j=0;j<MAX_CHANNELS;j++)
	{

/* but first we must determine whether to fill with 0 or 1. */
/* For a channel which has never been accessed (i.e. output_level = 0), we */
/*  fill with 0. Otherwise, we fill using the value of the last bit of the */
/*  channel. */
	  valu = 0;
	  mask = 1 << (j+1);
	  if ( (lvl=output_level[j]) )
	    {
	      if (output[2*(lvl-1)] & mask)
		valu = 1;
	      else
		valu = 0;
	    }
/* Fill with 1 if appropriate ... */
	  if (valu)
	    {
	      mask = 1 << (j+1);
	      for (i=output_level[j];i<top;i++)
		output[2*i] |= mask;
	    }
/* else fill with 0. */
	  else
	    {
	      mask = ~ (1 << (j+1));
	      for (i=output_level[j];i<top;i++)
		output[2*i] &= mask;
	    }
/* And finally update the output_level for the channel. */
	  output_level[j] = top;
	}

     }
  return;
} 



/****  8/27/96--DR ****/

/*------------------------------------------------------------------*/
/* Generate memory bitstreams from a .ini file */
/*                                                                  */
/* Allows one parameter in ini file to be set to custom value:      */
/* the routine looks for the string param_name in the comment field */
/* after the value, and on the first match sets the value to param. */
/* ALSO, if the ini file exists in memory as a bitstream then the   */
/* old bitstream is deleted.                                        */
/*   */
/* 07/11/97: redundant code eliminated; this routine now just calls */
/*            bs_getini_change. the routine itself is kept for      */
/*            backwards compatability                               */
/*------------------------------------------------------------------*/
int bs_getini_modify(char *in_name, int store, char *param_name, \
		     unsigned int param)
{
  char *pname[1];
  unsigned int p[1];
  int i;

  pname[0] = param_name;
  p[0] = param;

  i = bs_getini_change(in_name,store,1,pname,p);

  return(i);

}

/******************************************************************/
/* Clear 1 bitstream file from memory    */
/*  Added back in 27-Aug-1996, Al Clark  */
/******************************************************************/
int bs_delete(char *bs_name)
{
  int id;
  struct BS_CONTENTS *temp,*temp2;

/* find b/s file id number...*/
  if ((id=bs_id(bs_name))==-1)
    {
      printf("bs_delete: could not find %s.\n",bs_name);
      return(1);
    }

/* and remove from memory... */
  temp=bs_fileindex[id].first;
  while (temp!=NULL)
    {
      temp2=temp->next;
      free(temp);
      temp=temp2;
    }
/* and finally consolidate bs_fileindex.*/
  strcpy(bs_fileindex[id].name,bs_fileindex[next_file-1].name);
  bs_fileindex[id].first=bs_fileindex[next_file-1].first;
  next_file--;
  return(0);
}


/*------------------------------------------------------------------*/
/* Generate memory bitstreams from a .ini file */
/*                                                                   */
/* Allows arbitrary number of parameters in ini file to be changed.  */
/* Parameters:                                                       */
/*  in_name:      name of .ini file (without ".ini" suffix)          */
/*  store:        normally zero; set non-zero to store BN in the     */
/*                 DAQ fifo as the initialization pattern is strobed */
/*                 into the chip (pretty useless really)             */
/*  nchange:      number of desired changes to ini file              */
/*  param_name:   array of pointers to character strings. The        */
/*                 routine searches the comment fields after each    */
/*                 value specified in the .ini file, attempting to   */
/*                 match any of the strings given in param_name.     */
/*                 If a match is found, then the corresponding value */
/*                 in param[] is substituted for the value in the    */
/*                 .ini file.                                        */
/*  param:        array of values to substitute into ini file        */
/*                                                                   */
/* So for instance if one wanted to change the "chip id" field to 20 */
/*  and the "counter modulo" field to 100 in the file "default.ini", */
/*  the appropriate function call would be                           */
/*                                                                   */
/*   char *param_name[2] = {"chip id","counter modulo"};             */
/*   unsigned int param[2] = {20,100};                               */
/*     ...                                                           */
/*   i = bs_getini_change("default",0,2,param_name,param);           */
/*                                                                   */
/* Also note that if the ini file already exists in memory as a bit- */
/*  stream, then the old bitstream is deleted.                       */
/*-------------------------------------------------------------------*/
int bs_getini_change(char *in_name, int store, int nchange, \
		     char *param_name[], unsigned int *param)
{
        char name[MAX_NAME_LEN], filename[120], *f, *g, word[80];
	FILE *fp;
        int li=0;                /*counts lines of text*/
        int bit_array[2000];      /*holds unpacked bit values*/
        int i,ibit,bstart,bend,nb,btotal; /*counting etc*/
        int msb,bval;            /*temp storage*/
	unsigned int dval;       /*temp storage*/
        int TN;                  /*TN channels id*/
        char c,token;            
	struct BS_CONTENTS *new,*old;
	struct PACK_PTR loc;
	char *I="bs_getini_change";

/* find top neighbor channel number */
	if ( (TN=chn_id("tn")) < 0 ) log_in(ERROR,I,\
	  "TN channel not defined. Cannot create init stream");

/* Initialize bit array */
        for (i=0;i<2000;i++) bit_array[i] = -1;
        btotal = 0;

/* Standardize name */
        name_format( in_name, name );

/* Check that it does not already exist */
        if ( bs_id(name)!=-1 )
	  {
#ifndef QUIET
	    log_in(INFO,I,"name %s already exists--deleting",name);
#endif
	    bs_delete(name);       
	  }

/* Compose file name */
        filename[0] = '\0';
        f = strcpy( filename, file_path );
        f = strcat( filename, name );
        f = strcat( filename, ".ini"); 
#ifndef QUIET
        log_in(INFO,I,"reading data from %s",filename);
#endif


/* Open file */
	if ((fp=fopen(filename,"r"))==NULL)
	  {
	     log_in(WARNING,I,"could not open %s",filename);
	     return(1);
	   }

/* Begin loop over characters in ASCII file */
  	while (!feof(fp))
	  {
	    li++;

/* skip leading blanks in current line */
	    do  c=getc(fp); while ( isspace(c) );
	    if ( feof(fp) || (c=='\n') )  continue;

/* if first character is a ! then line is a comment. Ignore.*/
	     if ( c=='!')
               { 
                 while ( (c!='\n') && !feof(fp) ) c=getc(fp);
                 continue;
	       }
     
/* Line is not a comment */
/* Read ini bit descriptor */
	     i=0;
             token = '\0';
             bstart = -1;
             bend = -1;
             while ( c!=':' )
               {
                  if ( c>='0' && c<='9' )
                     word[i++] = c;
                  else if ( c=='-' || c=='<' || c=='>' )
	            {
                       token = c;
                       word[i] = '\0';
                       bstart = atoi(word);
                       i = 0;
	             }
                  else if ( c==' ');
                  else if ( c=='\n')
                    {
                       log_in(WARNING,I,"unexpected end of line");
		       fclose(fp);
                       return(2);
		     }
                  else if (feof(fp))
                    {
                       log_in(WARNING,I,"unexpected end of file");
		       fclose(fp);
                       return(3);
	             }
                  else
                    {
                       log_in(WARNING,I,"syntax error line %d, char. %c",li,c);
		       fclose(fp);
                       return(4);
	             }
                  c=getc(fp);
	        }
             word[i] = '\0';
             bend = atoi(word);

/* check if line is a single bit (no token)*/
             if ( token=='\0' && bstart==-1 )
               {
                if ( bit_array[bend]!=-1 ) 
                   {
                      log_in(WARNING,I,"line %d bit %d overwrite",li,bend);
                      fclose(fp);
                      return(5);
		    }

		i = fscanf(fp,"%d",&dval);

/* SEARCH for param_name; if found, set dval to param (single bit only) */
		if (nchange)
		  {

  /*Read in remainder of line*/
		    word[0] = '\0';
		    f = fgets(word,80,fp);

  /* Search for paramter name in line comment */
		    for (i=0;i<nchange;i++)
		      {
			if (param_name[i][0] == '\0')
			  continue;
			if ((g=strstr(word,param_name[i])) != NULL)
			  {
			    
  /* change single bit value */
			    dval = param[i];
			    if (dval) dval = 1;  

#ifndef QUIET
			    log_in(INFO,I,"Substituting %03d for %s.\n",\
				   dval,param_name[i]);
#endif
			  }
		      }
		  }

		bit_array[bend] = dval;
		btotal++;
		continue;
	      }

/* At this point we're looking at a range of bits.*/

/* check if range is sensible */
	    if ( (nb=bend-bstart+1)<=0 ) 
               {
                 log_in(WARNING,I,"line %d bad bit range",li);
                 fclose(fp);
                 return(6);
	       }
	    btotal += nb;

/* check no overwrite */
	    for (i=bstart;i<=bend;i++)
               if ( bit_array[i]!=-1 )
                   {
                      log_in(WARNING,I,"line %d bit %d overwrite",li,i);
                      return(7);
		    }

/*Must now decide data format*/

/*bitstream follows colon*/
             if ( token=='-' )
               {
                 ibit = bstart;
                 c = getc(fp);
                 while (!feof(fp) && c!='\n')
                   {          
                     bval = (c=='1');       
                     if ( bval || (c=='0') )
		       bit_array[ibit++] = bval;
                     c = getc(fp);
		   } 
                 if ( ibit!=(bend+1) )
                   {
                       log_in(WARNING,I,"last bit %d, not %d",\
                       (ibit-1),bend); 
                       return(8);
		    }
                 continue;
	        }

/* Data are decimal and must be converted.
   Read in decimal value to be converted to bits*/

/* A small hack to read in mask bits in arbitrary integer format.
   igv, 06/26/98 */
	     if (bend < 128)
	       i = fscanf(fp,"%i",&dval);
	     else
	       i = fscanf(fp,"%d",&dval);

/*Read in remainder of line*/
             word[0] = '\0';
             f = fgets(word,80,fp);

/* bit order. msb means most significant goes first*/
	    msb=(token=='>');
	    
/* gray or binary? Look for keyword gray*/
	    if ( (f=strstr(word,"gray"))==NULL )
	      if ( (f=strstr(word,"GRAY"))==NULL )
		f = strstr( word, "Gray" );

/* SEARCH for param_name; if found, set dval to param */
	    if (nchange)
	      {
		for (i=0;i<nchange;i++)
		  {
		    if (param_name[i][0] == '\0')
		      continue;
		    if ((g=strstr(word,param_name[i])) != NULL)
		      {
			dval = param[i];
#ifndef QUIET
			log_in(INFO,I,"Substituting %03d for %s.\n",\
			       param[i],param_name[i]);
#endif
		      }
		  }
	      }


/* carry out conversion */
	    if ( f==NULL )
	      dec_to_bin( dval, &bit_array[bstart], nb, msb );
	    else 
	      dec_to_gray( dval, &bit_array[bstart], nb, msb );


/*finished processing this line of text*/
	  }

/*finished processing file. Must now pack bits into memory linked list*/
/*first check that there are no gaps in bitsream*/
	fclose(fp);
        for (i=0;i<btotal;i++)
          if ( bit_array[i]==-1 )
            {
              log_in(WARNING,I,"bit %d unspecified",i);
              return(-1);
	    }

/*store filename*/
        f = strcpy( bs_fileindex[next_file].name, name );

/* Allocate memory for first record  and set local pointers*/
	bs_fileindex[next_file].first=\
           (struct BS_CONTENTS *)malloc(sizeof(struct BS_CONTENTS));
	old=bs_fileindex[next_file].first;
	new=old;
        ibit = 0;



/*start by putting a few zero bits*/ 
        old = bs_fillup( new, "TN", 6, "0" );
        new = old->next;

/* Each data bit will occupy 3 pattern bits (2 with clock low and
   one with clock high). Let each record hold 25 data bits, which
   will be a total of 75 bits*/
        for (i=0;i<=(btotal/25);i++) 
	  {
             loc.word=0;
             loc.bit=0;
             new->nBits=0;
             new->channel = TN;

/* transfer next 25 data bits */
             while ( (ibit<btotal) && (new->nBits<75) )
		 {
                       set_pack_bit(new->data,&loc,bit_array[ibit]);
		       pack_incr(&loc);
                       set_pack_bit(new->data,&loc,bit_array[ibit]);
		       pack_incr(&loc);
                       set_pack_bit(new->data,&loc,bit_array[ibit++]);
		       pack_incr(&loc);
		       new->nBits+=3;
		 }

/* Allocate next data record */
             old = new;
	     old->next=\
              (struct BS_CONTENTS *)malloc(sizeof(struct BS_CONTENTS));
	     new=old->next;
	   }

/* Now fill and allocate records for the clock clock and bits. */
/*start by putting a few zero bits*/
        old = bs_fillup( new, "ini_clock", 6, "0" ); 
        new = old->next;
        old = bs_fillup( new, "store", 6, "1" );
	new = old->next;

/*then add btotal 3-bit long clock cycles*/
        old = bs_fillup( new, "ini_clock", btotal, "010" ); 
        new = old->next;
        if (store) old = bs_fillup( new, "store", btotal+2, "101" );
        else old = bs_fillup( new, "store", btotal, "111" );
        new = old->next;

/* Fill bits for change mode*/
        old = bs_fillup( new, "chmod", 1, "111000" );
        new = old->next;

/* Fill bits for sr_load */
/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* Note that sr_load is longer than previous streams. Remember
   to bs_pad_level always after adding an ini bitstream set to the 
   curent pattern. This padding will also fill bits that NEED
   TO BE ZERO, such as mode1 and mode0. */
/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
        old = bs_fillup( new, "sr_load", (btotal+2), "000" );
        new = old->next;
        old = bs_fillup( new, "sr_load", 1, "01110" );
        new = old->next;

/*fill bits for preamp reset*/
        old = bs_fillup( new, "pa_reset", (btotal+3), "111" );
            
/* Success. clean up and return */
	free(old->next);
	old->next=NULL;
	next_file++;
#ifndef QUIET
        log_in(INFO,I,"done: %d lines, %d bits",li,btotal);
#endif
	return(0);
}




/*------------------------------------------------------------------*/
/* bs_appini_change: Generate memory bitstreams from a .ini file    */
/*                                                                   */
/*  Different from bs_addini_change in that the "change mode" and    */
/*   "sr_load" lines are not filled. This allows the repeated use    */
/*   of appini_change in order to initialize several chips on a      */
/*   hybrid. Previously, the only way to initialize a multi-chip     */
/*   hybrid was to construct one extremely long ini file.            */
/*  Another change is that if the routine executes successfully,     */
/*   then it returns the number of bits that will be loaded into     */
/*   the shift register (which is NOT the length of the bitstream    */
/*   created by this routine). In case of error, a negative number   */
/*   will be returned.                                               */
/*                                                                   */
/*                                                                   */
/* Allows arbitrary number of parameters in ini file to be changed.  */
/* Parameters:                                                       */
/*  in_name:      name of .ini file (without ".ini" suffix)          */
/*  store:        normally zero; set non-zero to store BN in the     */
/*                 DAQ fifo as the initialization pattern is strobed */
/*                 into the chip (pretty useless really)             */
/*  nchange:      number of desired changes to ini file              */
/*  param_name:   array of pointers to character strings. The        */
/*                 routine searches the comment fields after each    */
/*                 value specified in the .ini file, attempting to   */
/*                 match any of the strings given in param_name.     */
/*                 If a match is found, then the corresponding value */
/*                 in param[] is substituted for the value in the    */
/*                 .ini file.                                        */
/*  param:        array of values to substitute into ini file        */
/*                                                                   */
/* So for instance if one wanted to change the "chip id" field to 20 */
/*  and the "counter modulo" field to 100 in the file "default.ini", */
/*  the appropriate function call would be                           */
/*                                                                   */
/*   char *param_name[2] = {"chip id","counter modulo"};             */
/*   unsigned int param[2] = {20,100};                               */
/*     ...                                                           */
/*   i = bs_getini_change("default",0,2,param_name,param);           */
/*                                                                   */
/* Also note that if the ini file already exists in memory as a bit- */
/*  stream, then the old bitstream is deleted.                       */
/*-------------------------------------------------------------------*/
int bs_appini_change(char *in_name, int store, int nchange, \
		     char *param_name[], unsigned int *param)
{
        char name[MAX_NAME_LEN], filename[120], *f, *g, word[80];
	FILE *fp;
        int li=0;                /*counts lines of text*/
        int bit_array[800];      /*holds unpacked bit values*/
        int i,ibit,bstart,bend,nb,btotal; /*counting etc*/
        int msb,bval;            /*temp storage*/
	unsigned int dval;       /*temp storage*/
        int TN;                  /*TN channels id*/
        char c,token;            
	struct BS_CONTENTS *new,*old;
	struct PACK_PTR loc;
	char *I="bs_appini_change";

/* find top neighbor channel number */
	if ( (TN=chn_id("tn")) < 0 ) log_in(ERROR,I,\
	  "TN channel not defined. Cannot create init stream");

/* Initialize bit array */
        for (i=0;i<800;i++) bit_array[i] = -1;
        btotal = 0;

/* Standardize name */
        name_format( in_name, name );

/* Check that it does not already exist */
        if ( bs_id(name)!=-1 )
	  {
#ifndef QUIET
	    log_in(INFO,I,"name %s already exists--deleting",name);
#endif
	    bs_delete(name);       
	  }

/* Compose file name */
        filename[0] = '\0';
        f = strcpy( filename, file_path );
        f = strcat( filename, name );
        f = strcat( filename, ".ini"); 
#ifndef QUIET
        log_in(INFO,I,"reading data from %s",filename);
#endif


/* Open file */
	if ((fp=fopen(filename,"r"))==NULL)
	  {
	     log_in(WARNING,I,"could not open %s",filename);
	     return(-1);
	   }

/* Begin loop over characters in ASCII file */
  	while (!feof(fp))
	  {
	    li++;

/* skip leading blanks in current line */
	    do  c=getc(fp); while ( isspace(c) );
	    if ( feof(fp) || (c=='\n') )  continue;

/* if first character is a ! then line is a comment. Ignore.*/
	     if ( c=='!')
               { 
                 while ( (c!='\n') && !feof(fp) ) c=getc(fp);
                 continue;
	       }
     
/* Line is not a comment */
/* Read ini bit descriptor */
	     i=0;
             token = '\0';
             bstart = -1;
             bend = -1;
             while ( c!=':' )
               {
                  if ( c>='0' && c<='9' )
                     word[i++] = c;
                  else if ( c=='-' || c=='<' || c=='>' )
	            {
                       token = c;
                       word[i] = '\0';
                       bstart = atoi(word);
                       i = 0;
	             }
                  else if ( c==' ');
                  else if ( c=='\n')
                    {
                       log_in(WARNING,I,"unexpected end of line");
		       fclose(fp);
                       return(-2);
		     }
                  else if (feof(fp))
                    {
                       log_in(WARNING,I,"unexpected end of file");
		       fclose(fp);
                       return(-3);
	             }
                  else
                    {
                       log_in(WARNING,I,"syntax error line %d, char. %c",li,c);
		       fclose(fp);
                       return(-4);
	             }
                  c=getc(fp);
	        }
             word[i] = '\0';
             bend = atoi(word);

/* check if line is a single bit (no token)*/
             if ( token=='\0' && bstart==-1 )
               {
                if ( bit_array[bend]!=-1 ) 
                   {
                      log_in(WARNING,I,"line %d bit %d overwrite",li,bend);
                      fclose(fp);
                      return(-5);
		    }

		i = fscanf(fp,"%d",&dval);

/* SEARCH for param_name; if found, set dval to param (single bit only) */
		if (nchange)
		  {

  /*Read in remainder of line*/
		    word[0] = '\0';
		    f = fgets(word,80,fp);

  /* Search for paramter name in line comment */
		    for (i=0;i<nchange;i++)
		      {
			if (param_name[i][0] == '\0')
			  continue;
			if ((g=strstr(word,param_name[i])) != NULL)
			  {
			    
  /* change single bit value */
			    dval = param[i];
			    if (dval) dval = 1;  

#ifndef QUIET
			    log_in(INFO,I,"Substituting %03d for %s.\n",\
				   dval,param_name[i]);
#endif
			  }
		      }
		  }

		bit_array[bend] = dval;
		btotal++;
		continue;
	      }

/* At this point we're looking at a range of bits.*/

/* check if range is sensible */
	    if ( (nb=bend-bstart+1)<=0 ) 
               {
                 log_in(WARNING,I,"line %d bad bit range",li);
                 fclose(fp);
                 return(-6);
	       }
	    btotal += nb;

/* check no overwrite */
	    for (i=bstart;i<=bend;i++)
               if ( bit_array[i]!=-1 )
                   {
                      log_in(WARNING,I,"line %d bit %d overwrite",li,i);
                      return(-7);
		    }

/*Must now decide data format*/

/*bitstream follows colon*/
             if ( token=='-' )
               {
                 ibit = bstart;
                 c = getc(fp);
                 while (!feof(fp) && c!='\n')
                   {          
                     bval = (c=='1');       
                     if ( bval || (c=='0') )
		       bit_array[ibit++] = bval;
                     c = getc(fp);
		   } 
                 if ( ibit!=(bend+1) )
                   {
                       log_in(WARNING,I,"last bit %d, not %d",\
                       (ibit-1),bend); 
                       return(-8);
		    }
                 continue;
	        }

/* Data are decimal and must be converted.
   Read in decimal value to be converted to bits*/

/* A small hack to read in mask bits in arbitrary integer format.
   igv, 06/26/98 */
	     if (bend < 128)
	       i = fscanf(fp,"%i",&dval);
	     else
	       i = fscanf(fp,"%d",&dval);

/*Read in remainder of line*/
             word[0] = '\0';
             f = fgets(word,80,fp);

/* bit order. msb means most significant goes first*/
	    msb=(token=='>');
	    
/* gray or binary? Look for keyword gray*/
	    if ( (f=strstr(word,"gray"))==NULL )
	      if ( (f=strstr(word,"GRAY"))==NULL )
		f = strstr( word, "Gray" );

/* SEARCH for param_name; if found, set dval to param */
	    if (nchange)
	      {
		for (i=0;i<nchange;i++)
		  {
		    if (param_name[i][0] == '\0')
		      continue;
		    if ((g=strstr(word,param_name[i])) != NULL)
		      {
			dval = param[i];
#ifndef QUIET
			log_in(INFO,I,"Substituting %03d for %s.\n",\
			       param[i],param_name[i]);
#endif
		      }
		  }
	      }


/* carry out conversion */
	    if ( f==NULL )
	      dec_to_bin( dval, &bit_array[bstart], nb, msb );
	    else 
	      dec_to_gray( dval, &bit_array[bstart], nb, msb );


/*finished processing this line of text*/
	  }

/*finished processing file. Must now pack bits into memory linked list*/
/*first check that there are no gaps in bitsream*/
	fclose(fp);
        for (i=0;i<btotal;i++)
          if ( bit_array[i]==-1 )
            {
              log_in(WARNING,I,"bit %d unspecified",i);
              return(-9);
	    }

/*store filename*/
        f = strcpy( bs_fileindex[next_file].name, name );

/* Allocate memory for first record  and set local pointers*/
	bs_fileindex[next_file].first=\
           (struct BS_CONTENTS *)malloc(sizeof(struct BS_CONTENTS));
	old=bs_fileindex[next_file].first;
	new=old;
        ibit = 0;


/*start by putting a zero bit*/ 
        old = bs_fillup( new, "TN", 1, "0" );
        new = old->next;

/* Each data bit will occupy 3 pattern bits (2 with clock low and
   one with clock high). Let each record hold 25 data bits, which
   will be a total of 75 bits*/
        for (i=0;i<=(btotal/25);i++) 
	  {
             loc.word=0;
             loc.bit=0;
             new->nBits=0;
             new->channel = TN;

/* transfer next 25 data bits */
             while ( (ibit<btotal) && (new->nBits<75) )
		 {
                       set_pack_bit(new->data,&loc,bit_array[ibit]);
		       pack_incr(&loc);
                       set_pack_bit(new->data,&loc,bit_array[ibit]);
		       pack_incr(&loc);
                       set_pack_bit(new->data,&loc,bit_array[ibit++]);
		       pack_incr(&loc);
		       new->nBits+=3;
		 }

/* Allocate next data record */
             old = new;
	     old->next=\
              (struct BS_CONTENTS *)malloc(sizeof(struct BS_CONTENTS));
	     new=old->next;
	   }

/* Now fill and allocate records for the clock clock and bits. */
/*start by putting a zero bit*/
        old = bs_fillup( new, "ini_clock", 1, "0" ); 
        new = old->next;
        old = bs_fillup( new, "store", 1, "1" );
	new = old->next;

/*then add btotal 3-bit long clock cycles*/
        old = bs_fillup( new, "ini_clock", btotal, "010" ); 
        new = old->next;
        if (store) old = bs_fillup( new, "store", btotal, "101" );
        else old = bs_fillup( new, "store", btotal, "111" );
        new = old->next;

/* Success. clean up and return */
	free(old->next);
	old->next=NULL;
	next_file++;
#ifndef QUIET
        log_in(INFO,I,"done: %d lines, %d bits",li,btotal);
#endif
	return(btotal);
}




