/*---------------------------------------------------------------------*/
/*                    PATT    PACKAGE                                  */
/*                                                                     */ 
/*  chn.c: outines to acces channel name definition information        */
/* 7/18/96 changed routines to use logger instead of prinf statements  */
/*---------------------------------------------------------------------*/

#include <string.h>
#include "io.h"
#include "patt.h"

/*---------------------------------------------------------------------*/
/*	chn_clear -- clears channel name lookup table	*/
/*---------------------------------------------------------------------*/
void chn_clear(void)
{
	int i;
	for (i=0;i<MAX_CH_NAMES;i++)
	    channel[i].name[0] = '\0';	
}

/*---------------------------------------------------------------------*/
/*	chn_id -- returns # of channel with name ch_name.
		  Returns -i if ch_name not found, where i is the
                  number of the first free channel		       */
/*---------------------------------------------------------------------*/
int chn_id(char *ch_name)
{
	int i;
        char temp[MAX_NAME_LEN];

        i = 0;
      	name_format(ch_name,temp);        
        while ( channel[i].name[0] != '\0') 
	  {
            if (!strcmp(channel[i].name,temp)) 
		return(channel[i].number);
            i++;
	  }
	return(-i);
}

/*---------------------------------------------------------------------*/
/*   chn_load -- Loads in channel lookup table from disk file.
     NOTE:  NO ERROR CHECKING IMPLEMENTED WHILE READING!!            */
/*---------------------------------------------------------------------*/
int chn_load(char *name)
{
	FILE *fp;
	unsigned short chn;
	int j=0; 
	char *f,c,filename[80],temp[MAX_NAME_LEN];
	char *I="chn_load";

/* Compose file name */
        filename[0] = '\0';
        f = strcpy( filename, file_path );
        f = strcat( filename, name );
        f = strcat( filename, ".chn"); 

/*Open file*/
	if ((fp=fopen(filename,"r"))==NULL)
	  {
	    log_in(WARNING,I,"cannot open input file %s",filename);
	    return(1);
	   }
        else 
	    log_in(INFO,I,"reading from file %s",filename);

/*Read in definitions from file*/
	while (!feof(fp))
	   {
                if ( (c=getc(fp))=='!' )
		  {
		    while ( (c!='\n') && !feof(fp) ) c=getc(fp);
		    continue;
		  }
                else ungetc(c,fp);
		if (fscanf(fp,"%hu ",&chn)==0) continue;
		fscanf(fp,"%s\n",temp);
                j = chn_id(temp); 
                if ( j<0 ) j=-j;
                if (j>MAX_CH_NAMES) log_in(ERROR,I,\
      "array full. Increase MAX_CH_NAMES constant and remake executable");
                name_format( temp, channel[j].name );
                channel[j].number = chn;
	    }


/* close file*/
	fclose(fp);
	return(0);
}

/*---------------------------------------------------------------------*/
/*	chn_print -- prints out channel lookup table	*/
/*---------------------------------------------------------------------*/
void chn_print(void)
{
	int j=0;

	printf("\nChannels with names defined:\n");
 
        while ( channel[j].name[0] != '\0' )
 	   {
		printf("%3d:  ",channel[j].number);
		printf("%s\n ",channel[j++].name);
	   }
}



