/*---------------------------------------------------------*/
/*                     PATT   PACKAGE                      */
/*                                                         */
/*           generic.c: generic functions to simplify      */
/*                      user interface to patt package     */
/*---------------------------------------------------------*/

#include "io.h"
#include "patt.h"

/*--------------------------------------------------------------*/
/* Add a bitsream set to output pattern. If bitsream set is not */
/* in memory then load it first, then add it.                   */
/*--------------------------------------------------------------*/
int addpat(char *name)
{
  int test;
  char *I="addpat";

  if ( (test=bs_output(name)) == -1 ) 
    {
      if ( bs_loadmem(name) == 1 )
	{
	  log_in(WARNING,I,"can't find bitsrteam set %s",name);
	  return(-1);
	}
      else if ( bs_output(name) )
	log_in(ERROR,I,"something wrong with bitstream %s",name);
    }
  else if ( test )
    log_in(ERROR,I,"something wrong adding bitstream %s",name);
  return(0);
}

/*--------------------------------------------------------------*/
/* Add initalization bitsream set to output pattern. If bs is   */
/* not in memory then load it first, then add it.               */
/*--------------------------------------------------------------*/
int addini(char *name)
{
  int test;
  char *I="addini";

  if ( (test=bs_output(name)) == -1 ) 
    {
      if ( bs_getini(name,0) == 1 )
	{
	  log_in(WARNING,I,"can't find init file %s",name);
	  return(-1);
	}
      else if ( bs_output(name) )
	log_in(ERROR,I,"something wrong with bitstream %s",name);
    }
  else if ( test )
    log_in(ERROR,I,"something wrong adding bitstream %s",name);
  bs_pad_level("");
  return(0);
}


/*--------------------------------------------------------------*/
/* Add initalization bitsream set to output pattern. If bs is   */
/* not in memory then load it first, then add it.               */
/*--------------------------------------------------------------*/
int addini_modify(char *name, char *param_name, unsigned int param)
{
  int test;
  char *I="addini_modify";

/*
  if ( (test=bs_output(name)) == -1 ) 
    {
*/

      if ( bs_getini_modify(name,0,param_name,param) == 1 )
	{
	  log_in(WARNING,I,"can't find init file %s",name);
	  return(-1);
	}
      else if ( bs_output(name) )
	log_in(ERROR,I,"something wrong with bitstream %s",name);
/*
    }


  else if ( test )
    log_in(ERROR,I,"something wrong adding bitstream %s",name);
*/

  bs_pad_level("");
  return(0);
}

/*--------------------------------------------------------------*/
/* Add initalization bitsream set to output pattern. If bs is   */
/* not in memory then load it first, then add it.               */
/*--------------------------------------------------------------*/
int addini_change(char *name, int nchange, char *param_name[], \
		  unsigned int *param)
{
  int i;
  char *I="addini_change";

  i = bs_getini_change(name,0,nchange,param_name,param);

  if ( i == 1 )
    {
      log_in(WARNING,I,"can't find init file %s",name);
      return(-1);
    }
  else if (i)
    {
      log_in(ERROR,I,"some problem loading init file %s, code=%d",name,i);
      return(-1);
    }

  i = bs_output(name);
  if (i)
    log_in(ERROR,I,"something wrong with bitstream %s",name);

  bs_pad_level("");
  return(0);
}


/*--------------------------------------------------------------*/
/* Add initalization bitsream set to output pattern. If bs is   */
/* not in memory then load it first, then add it.               */
/*--------------------------------------------------------------*/
int appini(char *name)
{
  int nbits=0;
  char *I="appini";

  if (( nbits=bs_appini_change(name,0,0,NULL,NULL)) < 0 )
    {
      log_in(WARNING,I,"error %d appending file %s",nbits,name);
      return(nbits);
    }
  else if ( bs_output(name) )
    log_in(ERROR,I,"something wrong with bitstream %s",name);

  return(nbits);
}


/*--------------------------------------------------------------*/
/* Add initalization bitsream set to output pattern. If bs is   */
/* not in memory then load it first, then add it.               */
/*--------------------------------------------------------------*/
int appini_modify(char *name, char *param_name, unsigned int param)
{
  int nbits=0;
  char *I="appini_modify";
  char *pname[1];
  unsigned int p[1];

  pname[0] = param_name;
  p[0] = param;

  if ( (nbits=bs_appini_change(name,0,1,pname,p)) < 0 )
    {
      log_in(WARNING,I,"error %d appending file %s",nbits,name);
      return(nbits);
    }
  else if ( bs_output(name) )
    log_in(ERROR,I,"something wrong with bitstream %s",name);

  return(nbits);
}

/*--------------------------------------------------------------*/
/* Add initalization bitsream set to output pattern. If bs is   */
/* not in memory then load it first, then add it.               */
/*--------------------------------------------------------------*/
int appini_change(char *name, int nchange, char *param_name[], \
		  unsigned int *param)
{
  int nbits=0;
  char *I="appini_change";

  if ((nbits=bs_appini_change(name,0,nchange,param_name,param)) < 0 )
    {
      log_in(WARNING,I,"error %d appending file %s",nbits,name);
      return(nbits);
    }
  else if ( bs_output(name) )
    log_in(ERROR,I,"something wrong with bitstream %s",name);

  return(nbits);
}
