#ifndef _MGS_IO_H_
#define _MGS_IO_H_
/*---------------------------------------------------------------------*/
/*                          DATA        PACKAGE                        */
/*                                 and                                 */
/*                           LOG        PACKAGE                        */
/*                                                                     */
/*      io.h:  global variables and function prototypes                */
/*---------------------------------------------------------------------*/


#include <stdio.h>

/*******************************/
/* buffer handling routines    */
/*******************************/                  

/* Sets heder information in memory. Note that the information is NOT  */
/* automatically put in the output file. Function save_buffer copies   */
/* the current header info to the header of a new output file record   */
int set_header(int w, int d, int t, int MHz, int ulen, unsigned short *user);

/* gets header info. of the current record in the buffer. */
int get_header(int *w,int *d,int *t,int *MHz, int *ulen, unsigned short *user,\
		int *day, int *mon, int *year);

/* stores contents of argument arrays in memory buffer as a distinct event */
int store_event(unsigned char *data, unsigned char *bit9, unsigned char *type,\
		int bit9id, unsigned long nwords);

/* fills arrays pointed to by args. with data from next event in buffer */
int get_event(char *data, char *bit9, char *type, int *bit9id,\
	      unsigned long *nwords, int *event_num);

/* returns number of events in buffer */
int buffer_size(void); 

/* resets get_event to look at first event in buffer */
void first_event(void);

/* invalidate buffer */
void kill_buffer(void);


/************************/
/* file output routines */
/************************/

/* sets the name of output file */
int set_output_file(char *name);

/* appends entire record in buffer to output file, and then invalidates
    the buffer. leaves output file closed when done. */
int save_buffer(void);

/* writes out the current contents of the buffer in text format to a file */
int print_buffer(FILE *fp);

/***********************/
/* file input routines */
/***********************/

/* sets the name of the input file */
int set_input_file(char *name);

/* fills buffer with next record from input file, overwriting current */
/*  contents. opens input file if necessary, and leaves it open. */
int next_from_file(void); 

/* rewinds input file pointer to beginning of file */
int rewind_input_file(void);

/* Search forward in file for a record with given header info, and load */
/* it into the memory buffer if found. Arguments are wafer, die, test  */
/* and clock frequency. The special value 0 serves as a wildcard       */
int file_search_forward(int w, int d, int t, int MHz);

/* skips forward in input file by a specified number of records */
int file_skip_forward(unsigned long nrec);

/* closes input file */
int close_input_file(void);

/***********************************************************/
/* arithmetic routines (can also be found in patt library) */
/***********************************************************/

/* dec_to_bin: converts decimal to array of n bits in binary encoding*/
void dec_to_bin( unsigned int dval, int *bits, int n, int order );

/* dec_to_gray: converts decimal to array of n bits in gray encoding*/
void dec_to_gray( unsigned int dval, int *bits, int n, int order );

/* converts an 8-bit gray code word to binary*/
unsigned char  gray_to_bin(unsigned char gray);



/*-------------------------------------*/
/*   LOG.C data types and routines     */
/*-------------------------------------*/

/**************************/
/* logger message classes */
/**************************/

#define INFO    1
#define WARNING 2
#define ERROR   4

/***********************/
/* logger action types */
/***********************/

#define A_LOGFILE         1
#define A_SCREEN          2
#define A_ABORT           4
#define A_NONE            0
#define A_ERROR           7
#define A_WARNING         3


/****************************/
/* Patch to change name of  */
/* two logger routines to   */
/* avoid conflict with      */
/* Berkeley DB database     */
/* 2/10/99                  */
/****************************/
#define log_open(x) mgs_log_open(x)
#define log_close(x) mgs_log_close(x)

/******************************/
/* Logger routines            */
/******************************/
int log_open(char *name);
int log_close(void);
int log_in(int error_level, char *name, char *format, ...);
void log_to_screen(int i);
int log_limit(char *fn_name, int limit);

#endif /* _MGS_IO_H_ */
