

/*-------------------------------------------------------------------------*/
/*                         MODULES     PACKAGE                             */
/*                                                                         */
/*      aps.c:   for Kinetic Systems 3388 GPIB interface                   */
/*               and ALESSI REL-2500 probe station                         */
/*-------------------------------------------------------------------------*/


#include <string.h>
#include "io.h"
#include "camac_sjc.h"
#include "camac_modules.h"

unsigned long static aps_slot = 15;

/*** 12/12/96: camac_open, camac_close calls removed (see camac_sjc.c) */

void aps_address(int slot)
{
  aps_slot = slot;
}

void aps_init(void)
{
	short data[10];
	int qres,fd;
	char *I="aps_init";

/* Open CAMAC crate controller */
/*
	camac_open(&fd);
*/


/* enable various control signals */
	camac_cmd(aps_slot,0,25,0,0,(void*)NULL,&qres); /*InterFace Clear */
	camac_cmd(aps_slot,0,26,0,0,(void*)NULL,&qres);  /* ATtentioN */
	camac_cmd(aps_slot,1,26,0,0,(void*)NULL,&qres);  /* Remote ENable */
	camac_cmd(aps_slot,4,26,0,0,(void*)NULL,&qres);  /*comm. inhibit */

	data[0]=0x5f;
	camac_cmd(aps_slot,0,16,0,1,data,&qres);  /* UNTalk all modules */
	data[0]=0x3f;
	camac_cmd(aps_slot,0,16,0,1,data,&qres);  /* UNListen all modules */

/* Close CAMAC crate controller */
/*
	camac_close(fd);
*/

	return;
}

int aps_send(char *text, char *message)
{
	short data[10];
	int fd,qres;
	int i,j,len;
	char *I="aps_send";


/* Open CAMAC crate controller */
/*
	camac_open(&fd);
*/

/* enable ATtentioN */
	camac_cmd(aps_slot,0,26,0,0,(void*)NULL,&qres);

/* make probe station a listener */
	data[0]=0x20+3;
	camac_cmd(aps_slot,0,16,0,1,data,&qres);

/* make GPIB interface module a talker */
	data[0]=0x40+1;
	camac_cmd(aps_slot,0,16,0,1,data,&qres);

/* check output ready */
	for (i=0;;i++)
	  {
	    camac_cmd(aps_slot,1,27,0,0,(void*)NULL,&qres);
	    if (qres) break;
	    if (i==APS_TIMEOUT) log_in(ERROR,I,"output not ready 1");
	  }
/*disable ATtentioN*/
	camac_cmd(aps_slot,0,24,0,0,(void*)NULL,&qres);

/* send out command string, 1 character at a time */ 
	for (i=0;;i++)
		{
		if (text[i]=='\0')  /* End of String (EOS) */
			{
			data[0]=10;  /* Carriage Return: EOS character */
			camac_cmd(aps_slot,0,16,0,1,data,&qres);
			break;
			}

		data[0]=(short)text[i];
		camac_cmd(aps_slot,0,16,0,1,data,&qres);

		camac_cmd(aps_slot,0,1,0,1,data,&qres); /* get status */
		if (data[0] & 32)   /* check for handshake error */
		  log_in(ERROR,I,"GPIB interface handshake error");
		}

/* check output ready */
	camac_cmd(aps_slot,1,27,0,0,(void*)NULL,&qres);
	if (!qres) log_in(ERROR,I,"output not ready 2");
	camac_cmd(aps_slot,0,26,0,0,(void*)NULL,&qres);  /* ATtentioN */

/* make GPIB interface module a listener */
	data[0]=0x20+1;
	camac_cmd(aps_slot,0,16,0,1,data,&qres);

/* make probe station a talker */
	data[0]=0x40+3;
	camac_cmd(aps_slot,0,16,0,1,data,&qres);

/* check output ready */
	camac_cmd(aps_slot,1,27,0,0,(void*)NULL,&qres);
	if (!qres) log_in(ERROR,I,"output not ready 3");

/*disable ATtentioN*/
	camac_cmd(aps_slot,0,24,0,0,(void*)NULL,&qres);  

/* now wait for the probe station to finish executing command and */
/*  reply. moving the platten can take as long as 10 sec, so we   */
/*  generously set the timeout value at 15 sec.                   */
	for (j=0;;j++)
	  {
/* check input ready to see if probe station has data byte ready. */
/*  (true if Q=1)  */
	    camac_cmd(aps_slot,0,27,0,0,(void*)NULL,&qres);
	    if (qres) break;
	    if (j==15)        /* time-out after 15 sec. */
	      log_in(ERROR,I,"timed out waiting for reply from probe station");
	    else              /* otherwise wait a second & try again */
	      sginap(100);
	  }


/* read in reply from probe station one char. at a time */
	for (i=0,j=0;;)
	  {
	    camac_cmd(aps_slot,0,0,0,1,data,&qres);  /* read character */

	    if (!qres)
	      {
		if (++j==APS_TIMEOUT)
		  {
		    printf("aps_send: read timeout.\n");
		    return(1);
		  }
		continue;
	      }
	    if (data[0]==10)  /* end of string? */
	      {
		message[i]='\0';
		break;
	      }
	    if (data[0]) message[i++]=(char)data[0];
	    j=0;
	  }

/*
	camac_close(fd);
*/

	return(0);  /* success */
}


int aps_send2(int addr, char *text, char *message)
{
  short data[10];
  int fd,qres;
  int i,j,len;
  char *I="aps_send2";

  /* enable ATtentioN */
  camac_cmd(aps_slot,0,26,0,0,(void*)NULL,&qres);

  /* make probe station a listener */
  data[0]=0x20+addr;
  camac_cmd(aps_slot,0,16,0,1,data,&qres);

  /* make GPIB interface module a talker */
  data[0]=0x40+1;
  camac_cmd(aps_slot,0,16,0,1,data,&qres);

  /* check output ready */
  for (i=0;;i++)
  {
    camac_cmd(aps_slot,1,27,0,0,(void*)NULL,&qres);
    if (qres) break;
    if (i==APS_TIMEOUT) log_in(ERROR,I,"output not ready 1");
  }
  /* disable ATtentioN */
  camac_cmd(aps_slot,0,24,0,0,(void*)NULL,&qres);

  /* send out command string, 1 character at a time */ 
  for (i=0;;i++)
  {
    if (text[i]=='\0')  /* End of String (EOS) */
    {
      data[0]=13;
      camac_cmd(aps_slot,0,16,0,1,data,&qres);

      data[0]=10; 
      camac_cmd(aps_slot,2,26,0,0,NULL,&qres);  /* Enable End or Identify  */
      camac_cmd(aps_slot,0,16,0,1,data,&qres);
      camac_cmd(aps_slot,2,24,0,0,NULL,&qres);  /* Disable End or Identify */
      break;
    }

    data[0]=(short)text[i];
    camac_cmd(aps_slot,0,16,0,1,data,&qres);

    camac_cmd(aps_slot,0,1,0,1,data,&qres); /* get status */
    if (data[0] & 32)   /* check for handshake error */
      log_in(ERROR,I,"GPIB interface handshake error");
  }

  if (text[strlen(text)-1] != '?')
    return(0);

  /* check output ready */
  camac_cmd(aps_slot,1,27,0,0,(void*)NULL,&qres);
  if (!qres) log_in(WARNING,I,"output not ready 2");
  camac_cmd(aps_slot,0,26,0,0,(void*)NULL,&qres);  /* ATtentioN */

  /* make GPIB interface module a listener */
  data[0]=0x20+1;
  camac_cmd(aps_slot,0,16,0,1,data,&qres);

  /* make probe station a talker */
  data[0]=0x40+addr;
  camac_cmd(aps_slot,0,16,0,1,data,&qres);

  /* check output ready */
  camac_cmd(aps_slot,1,27,0,0,(void*)NULL,&qres);
  if (!qres) log_in(WARNING,I,"output not ready 3");

  /* disable ATtentioN */
  camac_cmd(aps_slot,0,24,0,0,(void*)NULL,&qres);  

  /* now wait for the probe station to finish executing command and */
  /*  reply. moving the platten can take as long as 10 sec, so we   */
  /*  generously set the timeout value at 15 sec.                   */
  for (j=0;;j++)
  {
    /* check input ready to see if probe station has data byte ready. */
    /*  (true if Q=1)  */
    camac_cmd(aps_slot,0,27,0,0,(void*)NULL,&qres);
    if (qres) break;
    if (j==15)        /* time-out after 15 sec. */
      log_in(ERROR,I,"timed out waiting for reply");
    else              /* otherwise wait a second & try again */
      sginap(100);
  }

 /* read in reply from probe station one char. at a time */
  for (i=0,j=0;;)
  {
    camac_cmd(aps_slot,0,0,0,1,data,&qres);  /* read character */

    if (!qres)
    {
      if (++j==APS_TIMEOUT)
      {
	printf("aps_send2: read timeout.\n");
	return(1);
      }
      continue;
    }

/*     if (data[0]) */
/*       printf("got ascii code %d\n", data[0]); */

    if (data[0]==10)  /* end of string? */
    {
      message[i]='\0';
      break;
    }

    if (data[0])
      message[i++]=(char)data[0];

    j=0;
  }

  return(0);  /* success */
}
