#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <dac02.h>

/* The following are the DAC ranges. They may be changed with on-board */
/*   DIP switches                                                      */
#define DAC0_LOWLIMIT 0.0f   /* In Volts */
#define DAC0_HILIMIT 5.0f
#define DAC1_LOWLIMIT 0.0f
#define DAC1_HILIMIT 5.0f

int dac02_set_port(int dac_number, unsigned long ival)
{
  int cd;
  
  cd = -1;
  switch (dac_number)
  {
      case 0:
	cd = open("/dev/dac02/dac0", O_WRONLY);
	break;
      case 1:
	cd = open("/dev/dac02/dac1", O_WRONLY);
	break;
  }
  if (cd < 0)
  {
      fprintf(stderr, 
	      "ERROR in dac02_set_port: failed to open DAC %d port\n", 
	      dac_number);
      return -1;
  }
  else
  {
    write(cd, &ival, 2);
    close(cd);
    return 0;
  }
}

int dac02_set(int dac_number, float volts)
{
  unsigned long ival;

  switch (dac_number)
  { 
      case 0:
	if (volts < DAC0_LOWLIMIT || volts > DAC0_HILIMIT)
	{
	  fprintf(stderr,
		  "ERROR in dac02_set: %f Volts is out of DAC 0 range\n",
		  volts);
	  return -1;
	}
	ival = (int)((volts-DAC0_LOWLIMIT)/(DAC0_HILIMIT-DAC0_LOWLIMIT)*65536.0f);
	if (ival == 65536)
	  ival--;
	return dac02_set_port(dac_number, ival);
	break;

      case 1:
	if (volts < DAC1_LOWLIMIT || volts > DAC1_HILIMIT)
	{
	  fprintf(stderr,
		  "ERROR in dac02_set: %f Volts is out of DAC 1 range\n",
		  volts);
	  return -1;
	}
	ival = (int)((volts-DAC1_LOWLIMIT)/(DAC1_HILIMIT-DAC1_LOWLIMIT)*65536.0f);
	if (ival == 65536)
	  ival--;
	return dac02_set_port(dac_number, ival);
	break;

      default:
	fprintf(stderr, 
		"ERROR in dac02_set: DAC number '%d' is out of range\n",
		dac_number);
	return -1;
	break;
  }
}
