
/*-------------------------------------------------------------------------*/
/*                         MODULES     PACKAGE                             */
/*                                                                         */
/*      inreg.c:   for Kinetic Systems 3430 input register                 */
/*-------------------------------------------------------------------------*/

#include "camac_sjc.h"
#include "camac_modules.h"

/*** 12/12/96: camac_open & camac_close calls removed (see camac_sjc.c) */
/*** 12/13/96: data now read from input register in blocks of 600 words */
/*              rather than 128, to minimize camac calls.               */


#define BLOCK_SIZE 4096

unsigned long static inreg_slot = 17;
unsigned int static inreg_order = 1;

void inreg_address(int slot)
{
/* inreg_order selects which bytes (odd or even) of the input register are
   transferred to the array returned by inreg_read_fifo. Only half of the
   bytes contain actual information, while the other half are always junk
   due to the way the KS 3430 is wired up to the PATT board. Whether it's
   the odd or the even bytes that one wants depends on the polarity of the 
   differential "clock" signal going from the input register to the PATT 
   board. */

  if (slot<0) 
    {
      inreg_slot = -slot;
      inreg_order = 0;
    }
  else
    {
      inreg_slot = slot;
      inreg_order = 1;
    }
}

int inreg_read_fifo(unsigned char *array, unsigned char *bit8)
{
  int fd,qres,i,j=0,more=1;
  unsigned long subad=0;
  short k,mask;
  unsigned short data[4096],temp[2]={0,0};


  unsigned long state,trig[3];


/* reset input register address */
  data[0] = 0;
  camac_cmd(inreg_slot,subad,17,0,1,data,&qres);

  j = 0;

/* temporary "trigger" signal for examining data readback */
/*
  state = outreg0_getstate();
  trig[0] = state | (1<<6);
  trig[1] = trig[0];
  trig[2] = state;
  outreg(0,trig,3);
*/



  while ( more && (j < BLOCK_SIZE) )
    {
      camac_cmd(inreg_slot,subad,0,2,BLOCK_SIZE,data,&qres);

      if ( !qres )
	{
	  printf("inreg_read_fifo: short transfer! QRES=%d\n",qres);
	  getchar();
	}

      i = inreg_order;
      while ( more && (i < BLOCK_SIZE) ) 
	{
	  bit8[j]=((data[i] & 0x0100) ? 1 : 0 );
	  array[j]=(data[i] & 0x00ff);
	  j++;
	  more = data[i] & 0x0200;
	  i+=2;

	}

    }  

  return(j);
}







